from types import SimpleNamespace
from typing import Optional

from pydantic import BaseModel

PaginationDefaults = SimpleNamespace(starting_token=0, limit=25)


class PaginationRequestMixin(BaseModel):
    starting_token: int = PaginationDefaults.starting_token
    limit: int = PaginationDefaults.limit


class PaginationResponseMixin(PaginationRequestMixin, BaseModel):
    # Did we paginate?
    paginated: bool = False
    # If we paginated, what is the next starting token?
    # If we didn't paginate, this is None.
    next_starting_token: Optional[int] = None



from enum import Enum


class Scorers(str, Enum):
    toxicity = "toxicity"
    factuality = "factuality"
    groundedness = "groundedness"
    pii = "pii"
    latency = "latency"
    context_relevance = "context_relevance"



from typing import List, Optional, Tuple, Union

from promptquality.constants.scorers import Scorers
from promptquality.types.custom_scorer import CustomScorer


def bifurcate_scorers(
    scorers: Optional[List[Union[Scorers, CustomScorer]]] = None
) -> Tuple[List[Scorers], List[CustomScorer]]:
    if scorers is None:
        return [], []
    galileo_scorers = []
    custom_scorers = []
    for scorer in scorers:
        if isinstance(scorer, Scorers):
            galileo_scorers.append(scorer)
        elif isinstance(scorer, CustomScorer):
            custom_scorers.append(scorer)
        else:
            raise ValueError(f"Unknown scorer type: {type(scorer)}.")
    return galileo_scorers, custom_scorers



from typing import Any, Dict, List, Optional

from pydantic import UUID4, BaseModel, ConfigDict, Field

from promptquality.types.pagination import (
    PaginationRequestMixin,
    PaginationResponseMixin,
)


class GetRowsRequest(PaginationRequestMixin):
    project_id: UUID4
    run_id: UUID4

    def params(self) -> Dict[str, Any]:
        """
        Params to be passed to the API request.

        These are primarily the pagination parameters.

        Returns
        -------
        Dict[str, Any]
            Params to be passed to the API request.

        """
        return self.model_dump(mode="json", exclude={"project_id", "run_id"})


class Metrics(BaseModel):
    hallucination: Optional[float] = None
    bleu: Optional[float] = None
    rouge: Optional[float] = None
    pii: Optional[List[str]] = Field(default_factory=list)
    like_dislike: Optional[bool] = None
    toxicity: Optional[float] = None
    factuality: Optional[float] = None
    groundedness: Optional[float] = None
    latency: Optional[float] = None
    context_relevance: Optional[float] = None

    model_config = ConfigDict(extra="allow")


class PromptRow(BaseModel):
    index: int
    prompt: Optional[str] = None
    response: Optional[str] = None
    target: Optional[str] = None
    inputs: Dict[str, Optional[Any]] = Field(default_factory=dict)
    hallucination: Optional[float] = None
    bleu: Optional[float] = None
    rouge: Optional[float] = None
    cost: Optional[float] = None
    like_dislike: Optional[bool] = None
    metrics: Metrics = Field(default_factory=Metrics)

    model_config = ConfigDict(extra="allow")


class PromptRows(PaginationResponseMixin):
    rows: List[PromptRow] = Field(default_factory=list)



from random import random
from time import sleep
from typing import Optional

from pydantic import UUID4
from tqdm.auto import tqdm

from promptquality.constants.job import JobStatus
from promptquality.helpers import get_job_status
from promptquality.types.config import Config
from promptquality.utils.logger import logger


def job_progress(
    job_id: Optional[UUID4] = None, config: Optional[Config] = None
) -> UUID4:
    backoff = random()
    job_status = get_job_status(job_id, config)
    job_progress_bar = tqdm(
        total=job_status.steps_total,
        position=0,
        leave=True,
        desc=job_status.progress_message,
    )
    while job_status.status in [JobStatus.unstarted, JobStatus.in_progress]:
        sleep(backoff)
        job_status = get_job_status(job_id, config)
        job_progress_bar.set_description(job_status.progress_message)
        job_progress_bar.update(job_status.steps_completed - job_progress_bar.n)
        backoff = random()
    job_progress_bar.close()
    logger.debug(f"Job {job_id} status: {job_status.status}.")
    return job_status.id
